@@ -2,8 +2,8 @@
 #
 # summary.scantwo.R
 #
-# copyright (c) 2001-2014, Karl W Broman, Hao Wu, and Brian Yandell
-# last modified Oct, 2014
+# copyright (c) 2001-2019, Karl W Broman, Hao Wu, and Brian Yandell
+# last modified Dec, 2019
 # first written Nov, 2001
 #
 #     This program is free software; you can redistribute it and/or
@@ -36,10 +36,10 @@ summary.scantwo <-
     function(object, thresholds,
              what=c("best", "full", "add", "int"),
              perms, alphas, lodcolumn=1, pvalues=FALSE,
-             df=FALSE, allpairs=TRUE, ...)
+             allpairs=TRUE, ...)
 {
-    if(!any(class(object) == "scantwo") &&
-       !any(class(object) == "scantwocondensed"))
+    if(!inherits(object, "scantwo") &&
+       !inherits(object, "scantwocondensed"))
         stop("Input should have class \"scantwo\".")
 
     addpair <- attr(object, "addpair")
@@ -133,17 +133,24 @@ summary.scantwo <-
             stop("alphas should be a numeric vector")
     }
 
-    if(!missing(perms) && !any(class(perms) == "scantwoperm"))
+    if(!missing(perms) && !inherits(perms, "scantwoperm"))
         stop("perms must be in scantwoperm format.")
 
     # subset object and permutations, if necessary
-    if(any(class(object) == "scantwo")) {
+    if(inherits(object, "scantwo")) {
         d <- dim(object$lod)
         if(length(d)==3) {
             if(!missing(perms)) {
-                ncp <- sapply(perms, ncol)
+                if("AA" %in% names(perms)) { # contains X-specific results
+                    ncp <- sapply(perms$AA, ncol)
+                }
+                else {
+                    ncp <- sapply(perms, ncol)
+                }
+
                 if(all(ncp==1)) onepermcol <- TRUE
                 else onepermcol <- FALSE
+
                 if(any(ncp != d[3])) {
                     if(onepermcol)  {
                         if(lodcolumn > 1)
@@ -158,8 +165,15 @@ summary.scantwo <-
                 stop("lodcolumn must be between 1 and ", d[3])
 
             object$lod <- object$lod[,,lodcolumn]
-            if(!missing(perms) && !onepermcol)
-                perms <- lapply(perms, function(a, b) a[,b,drop=FALSE], lodcolumn)
+            if(!missing(perms) && !onepermcol) {
+                if("AA" %in% names(perms)) {
+                    for(i in seq(along=perms))
+                        perms[[i]] <- lapply(perms[[i]], function(a, b) a[,b,drop=FALSE], lodcolumn)
+                }
+                else {
+                    perms <- lapply(perms, function(a, b) a[,b,drop=FALSE], lodcolumn)
+                }
+            }
         }
     }
     else { # condensed version
@@ -202,7 +216,7 @@ summary.scantwo <-
         warning("p-values may be calculated only if perms are provided.")
     }
 
-    if(any(class(object) == "scantwo"))
+    if(inherits(object, "scantwo"))
         out <- subrousummaryscantwo(object, for.perm=FALSE)
     else
         out <- as.data.frame(object)
@@ -268,8 +282,8 @@ summary.scantwo <-
 
     if(!missing(perms) && "AA" %in% names(perms)) {
         xchr_specific <- TRUE
-        chrtype <- attr(perms, "chrtype")
-        chrpair_type <- paste0(chrtype[out$chr1], chrtype[out$chr2])
+        chr_type <- attr(perms, "chrtype")
+        chrpair_type <- paste0(chr_type[out$chr1], chr_type[out$chr2])
         if(all(chrpair_type=="AA")) {
             perms <- perms$AA
             xchr_specific <- FALSE
@@ -354,19 +368,6 @@ summary.scantwo <-
         out <- result
     }
 
-    attr(out, "df") <- revisescantwodf(attr(object, "df"))
-
-    if(!missing(perms) && "df" %in% names(attributes(out)) &&
-       "df" %in% names(attributes(perms))) {
-        df.o <- attr(out, "df")
-        df.p <- attr(perms, "df")
-        if(length(df.o) != length(df.p) ||
-           any(df.o != df.p))
-            warning("Degrees of freedom in input object and perms do not match.")
-    }
-
-    if(!df) attr(out, "df") <- NULL
-
     class(out) <- c("summary.scantwo", "data.frame")
 
     out
@@ -509,14 +510,6 @@ print.summary.scantwo <-
         return(invisible(NULL))
     }
 
-    if("df" %in% names(attributes(x))) {
-        df <- attr(x, "df")
-        cat("Degrees of freedom:\n")
-        rownames(df) <- paste("   ", rownames(df), ": ", sep="")
-        print(df)
-        cat("\n")
-    }
-
     z <- as.character(unlist(x[,1]))
 
     if(max(nchar(z)) == 1)
@@ -599,10 +592,10 @@ print.scantwo <-
 max.scantwo <-
     function(object, lodcolumn=1,
              what=c("best", "full", "add", "int"),
-             df=FALSE, na.rm=TRUE, ...)
+             na.rm=TRUE, ...)
 {
-    if(class(object)[1] != "scantwo" &&
-       class(object)[1] != "scantwocondensed")
+    if(!inherits(object, "scantwo") &&
+       !inherits(object, "scantwocondensed"))
         stop("Input must have class \"scantwo\".")
 
     addpair <- attr(object, "addpair")
@@ -613,7 +606,7 @@ max.scantwo <-
     }
 
     what <- match.arg(what)
-    if(class(object)[1] == "scantwo") {
+    if(inherits(object, "scantwo")) {
         d <- dim(object$lod)
         if(length(d)==3) {
             if(lodcolumn < 1 || lodcolumn > d[3])
@@ -698,8 +691,6 @@ max.scantwo <-
 
     class(out) <- c("summary.scantwo", "data.frame")
     rownames(out) <- what
-    if(df && "df" %in% names(attributes(object)))
-        attr(out, "df") <- revisescantwodf(attr(object,"df"))
 
     out
 }
@@ -718,7 +709,7 @@ max.scantwo <-
 clean.scantwo <-
     function(object, n.mar=1, distance=0, ...)
 {
-    if(class(object)[1] != "scantwo")
+    if(!inherits(object, "scantwo"))
         stop("Input should have class \"scantwo\".")
 
     addpair <- attr(object, "addpair")
@@ -787,7 +778,7 @@ clean.scantwo <-
 subset.scantwo <-
     function(x, chr, lodcolumn, ...)
 {
-    if(class(x)[1] != "scantwo")
+    if(!inherits(x, "scantwo"))
         stop("Input should have class \"scantwo\".")
 
     if((missing(chr) || length(chr)==0) && missing(lodcolumn)) return(x)
@@ -830,10 +821,6 @@ subset.scantwo <-
             attr(x, "fullmap") <- fmap
         }
 
-        df <- attr(x, "df")
-        if(any(!is.na(match(c("AX","XX"), rownames(df)))) &&
-           all(!x$map$xchr))
-            attr(x, "df") <- df["AA",,drop=FALSE]
     }
 
     x
@@ -846,9 +833,9 @@ subset.scantwo <-
 # scantwo permutation test (from scantwo with n.perm > 0)
 ######################################################################
 summary.scantwoperm <-
-    function(object, alpha=c(0.05, 0.10), df=FALSE, ...)
+    function(object, alpha=c(0.05, 0.10), ...)
 {
-    if(!any(class(object) == "scantwoperm"))
+    if(!inherits(object, "scantwoperm"))
         stop("Input should have class \"scantwoperm\".")
 
     if("AA" %in% names(object)) { # X-chr-specific version
@@ -899,9 +886,6 @@ summary.scantwoperm <-
         rownames(out[[i]]) <- paste0(alpha*100, "%")
     }
 
-    if(df && "df" %in% names(attributes(object)))
-        attr(out, "df") <- attr(object, "df")
-
     attr(out, "n.perm") <- nrow(object[[1]])
     class(out) <- c("summary.scantwoperm", "list")
     out
@@ -951,14 +935,6 @@ print.summary.scantwoperm <-
         stop("The components shouldn't have varying numbers of columns.\n")
     nc <- nc[1]
 
-    if("df" %in% names(attributes(x))) {
-        df <- attr(x, "df")
-        cat("Degrees of freedom:\n")
-        rownames(df) <- paste("   ", rownames(df), ": ", sep="")
-        print(df)
-        cat("\n")
-    }
-
     if(nc==1) {
         phe <- colnames(x[[1]])
         if(is.null(phe)) phe <- ""
@@ -997,12 +973,11 @@ cbind.scantwo <- c.scantwo <-
 {
     dots <- list(...)
 
-    cl1 <- class(dots[[1]])
-    if(length(dots)==1 && length(cl1)==1 && cl1=="list") dots <- dots[[1]]
+    if(length(dots)==1 && is.list(dots[[1]])) dots <- dots[[1]]
 
     if(length(dots)==1) return(dots[[1]])
     for(i in seq(along=dots)) {
-        if(!any(class(dots[[i]]) == "scantwo"))
+        if(!inherits(dots[[i]], "scantwo"))
             stop("Input should have class \"scantwo\".")
     }
 
@@ -1063,14 +1038,6 @@ cbind.scantwo <- c.scantwo <-
         }
     }
 
-    df <- lapply(dots, attr, "df")
-    for(i in 2:length(df)) {
-        if(length(df[[1]]) != length(df[[i]]) || any(df[[1]] != df[[i]])) {
-            warning("Mismatch in degrees of freedom.")
-            attr(output, "df") <- NULL
-        }
-    }
-
     output
 }
 
@@ -1083,8 +1050,7 @@ rbind.scantwoperm <- c.scantwoperm <-
 {
     dots <- list(...)
 
-    cl1 <- class(dots[[1]])
-    if(length(dots)==1 && length(cl1)==1 && cl1=="list") dots <- dots[[1]]
+    if(length(dots)==1 && is.list(dots[[1]])) dots <- dots[[1]]
     if(length(dots)==1) return(dots[[1]])
 
     xchrsp <- vapply(dots, function(a) "AA" %in% names(a), TRUE)
@@ -1106,7 +1072,7 @@ rbind.scantwoperm <- c.scantwoperm <-
     }
 
     for(i in seq(along=dots)) {
-        if(!any(class(dots[[i]]) == "scantwoperm"))
+        if(!inherits(dots[[i]], "scantwoperm"))
             stop("Input should have class \"scantwoperm\".")
     }
 
@@ -1123,18 +1089,6 @@ rbind.scantwoperm <- c.scantwoperm <-
     }
     if(flag) warning("Mismatch in column names; input may not be consistent.\n")
 
-    df <- lapply(dots, attr, "df")
-    flag <- 0
-    for(i in 2:length(df)) {
-        if(length(df[[i]]) != length(df[[1]]) ||
-           any(df[[i]] != df[[1]]))
-            flag <- 1
-    }
-    if(flag) {
-        warning("Mismatch in degrees of freedom; may cause problems.")
-        attr(dots[[1]], "df") <- NULL
-    }
-
     dots[[1]]
 }
 
@@ -1143,8 +1097,7 @@ cbind.scantwoperm <-
 function(...)
 {
     dots <- list(...)
-    cl1 <- class(dots[[1]])
-    if(length(dots)==1 && length(cl1)==1 && cl1=="list") dots <- dots[[1]]
+    if(length(dots)==1 && is.list(dots[[1]])) dots <- dots[[1]]
 
     if(length(dots)==1) return(dots)
 
@@ -1188,7 +1141,6 @@ condense.scantwo <-
     function(object)
 {
     out <- subrousummaryscantwo(object, for.perm=FALSE)
-    attr(out, "df") <- attr(object, "df")
     class(out) <- c("scantwocondensed", "list")
     out
 }
@@ -1206,7 +1158,7 @@ subset.scantwoperm <-
     if("AA" %in% names(x)) { # x chr specific
         for(j in seq(along=x)) {
             if(missing(lodcolumn)) lodcolumn <- 1:ncol(x[[j]][[1]])
-            else if(!is.null(attr(try(x[[j]][[1]][,lodcolumn], silent=TRUE),"try-error")))
+            else if(!check_colindex(lodcolumn, x[[j]][[1]]))
                 stop("lodcolumn misspecified.")
 
             repl <- 1:nrow(x[[j]][[1]])
@@ -1222,11 +1174,11 @@ subset.scantwoperm <-
         x <- lapply(x, as.matrix)
 
     if(missing(lodcolumn)) lodcolumn <- 1:ncol(x[[1]])
-    else if(!is.null(attr(try(x[[1]][,lodcolumn], silent=TRUE),"try-error")))
+    else if(!check_colindex(lodcolumn, x[[1]]))
         stop("lodcolumn misspecified.")
 
     if(missing(repl)) repl <- 1:nrow(x[[1]])
-    else if(!is.null(attr(try(x[[1]][repl,], silent=TRUE),"try-error")))
+    else if(!check_rowindex(repl, x[[1]]))
         stop("repl misspecified.")
 
     cl <- class(x)
